import numpy as np
import pandas as pd
import time
import sys
from PIL import Image, ImageTk

import Tkinter as tk

UNIT = 100   # pixels
HALF_UNIT = UNIT/2
QUARTER_UNIT = HALF_UNIT/2
HALF_UNIT_MINUS10 = HALF_UNIT - 10
SIZE = int(3*UNIT/8)  #HALF_UNIT_MINUS10
np.random.seed(2)  # reproducible
ACTIONS = ['left', 'right','up','down']     # available actions
#AI Parameter
EPSILON = 0.9   # greedy police
ALPHA = 0.01     # learning rate
GAMMA = 0.9    # discount factor
MAX_EPISODES = 100   # maximum episodes

def build_maze():
    # Creation of the grid
    for c in range(0, MAZE_W * UNIT, UNIT):
        x0, y0, x1, y1 = c, 0, c, MAZE_H * UNIT
        canvas.create_line(x0, y0, x1, y1)
    for r in range(0, MAZE_H * UNIT, UNIT):
        x0, y0, x1, y1 = 0, r, MAZE_W * UNIT, r
        canvas.create_line(x0, y0, x1, y1)
    
    # Creation of the treasure
    """X, Y = get_coord(TREASURE-1)
                oval_center = origin + np.array([UNIT * X, UNIT * Y])
                oval = canvas.create_oval(
                                    oval_center[0] - SIZE, oval_center[1] - SIZE,
                                    oval_center[0] + SIZE, oval_center[1] + SIZE,
                                    fill='yellow')"""
    X, Y = get_coord(TREASURE-1)
    oval_center = origin + np.array([UNIT * X, UNIT * Y])
    canvas._photo1 = photo1 = ImageTk.PhotoImage(file= "image/Tresor.png")
    canvas.create_image(oval_center[0]-SIZE-5,oval_center[1]-SIZE+5,anchor=tk.NW,image=photo1)

    # Creation of walls
    canvas._photo2 = photo2 = ImageTk.PhotoImage(file= "image/Wall.png")
    for w in WALLS:
        X, Y = get_coord(w)
        rect_origin = origin + np.array([UNIT * X, UNIT * Y])
        rect = canvas.create_image(rect_origin[0]-50,rect_origin[1]-49,anchor=tk.NW,image=photo2)

    # Creation of traps
    canvas._photo3 = photo3 = ImageTk.PhotoImage(file= "image/Bombe.png")
    for t in TRAPS:
        X, Y = get_coord(t)
        oval_center = origin + np.array([UNIT * X, UNIT * Y])
        canvas.create_image(oval_center[0]-SIZE+7,oval_center[1]-SIZE+10,anchor=tk.NW,image=photo3)

    canvas.pack()

def get_coord(S):
    X = int(S%MAZE_W)
    Y = int(S/MAZE_W)
    return X, Y

def draw_rect(S):
    S = S - 1
    X, Y = get_coord(S)
    rect_origin = origin + np.array([UNIT * X, UNIT * Y])
    canvas._photo4 = photo4 = ImageTk.PhotoImage(file= "image/Agent.png")
    rect = canvas.create_image(rect_origin[0]-SIZE-12,rect_origin[1]-SIZE-10,anchor=tk.NW,image=photo4)
    return rect

def build_q_table(n_states, actions):
    table = pd.DataFrame(
        np.zeros((n_states, len(actions))),     # q_table initial values
        columns=actions,    # actions's name
    )
    #print(table)    # show table
    return table

def choose_action(state, q_table):
    # This is how to choose an action
    state_actions = q_table.iloc[state, :]
    if (np.random.uniform() > EPSILON) or ((state_actions == 0).all()):  # act non-greedy or state-action have no value
        action_name = np.random.choice(ACTIONS)
    else:   # act greedy
        action_name = state_actions.idxmax()    # replace argmax to idxmax as argmax means a different function in newer version of pandas
    return action_name

def get_env_feedback(S, A):
    # This is how agent will interact with the environment
    R = 0
    if S == TREASURE - 1:   # reach treasure
            S_ = 'terminal'
            R = 1
    else:
        for t in TRAPS: 
            if S == t:  # reach a trap
                S_ = 'terminal'
                R = -10
                break
        if R == 0:
            if A == 'right':    # move right
                if S % MAZE_W == MAZE_W - 1: # reach the right wall
                    S_ = S
                else:
                    S_ = S + 1
                    for w in WALLS: 
                        if S + 1 == w:  # reach a black wall
                            S_ = S
                            break            
            elif A == 'left':   # move left
                if S % MAZE_W == 0: # reach the left wall
                    S_ = S 
                else:
                    S_ = S - 1
                    for w in WALLS: 
                        if S - 1 == w:  # reach a black wall
                            S_ = S
                            break 
            elif A == 'up':   # move up
                if S - MAZE_W < 0: # reach the up wall
                    S_ = S 
                else:
                    S_ = S - MAZE_W
                    for w in WALLS: 
                        if S - MAZE_W == w:  # reach a black wall
                            S_ = S
                            break 
            else:   # move down
                if S + MAZE_W >= MAZE_W * MAZE_H: # reach the down wall
                    S_ = S 
                else:
                    S_ = S + MAZE_W
                    for w in WALLS: 
                        if S + MAZE_W == w:  # reach a black wall
                            S_ = S
                            break

    return S_, R

def update_env(S, episode, step_counter):
    global rectangle
    global canvas
    global q_table
    if S == 'terminal':
        print('Episode : total_steps = ', episode+1, step_counter)
        time.sleep(0.5)
    else:
        canvas.delete(rectangle)
        rectangle = draw_rect(S+1)
        canvas.update()
        time.sleep(FRESH_TIME)

def rl(AGENT):
    # main part of RL loop
    q_table = build_q_table(N_STATES, ACTIONS)
    print(q_table) 
    for episode in range(MAX_EPISODES):
        step_counter = 0
        S = AGENT
        is_terminated = False
        update_env(S, episode, step_counter)
        while not is_terminated:
            A = choose_action(S, q_table)
            S_, R = get_env_feedback(S, A)  # take action & get next state and reward
            q_predict = q_table.loc[S, A]
            if S_ != 'terminal':
                q_target = R + GAMMA * q_table.iloc[S_, :].max()   # next state is not terminal
            else:
                q_target = R     # next state is terminal
                is_terminated = True    # terminate this episode

            q_table.loc[S, A] += ALPHA * (q_target - q_predict)  # update
            S = S_  # move to next state

            update_env(S, episode, step_counter+1)
            step_counter += 1
            print(q_table)
        print(q_table)
    return q_table

if __name__ == "__main__":
    MAZE_H = 3   # grid height
    MAZE_W = 3   # grid width
    N_STATES = MAZE_H*MAZE_W  # Total of state
    WIDTH = MAZE_W*UNIT
    HEIGHT = MAZE_H*UNIT
    TREASURE = 8
    WALLS = [] # walls in the maze
    TRAPS = [] #13 28 34
    AGENT = 3
    FRESH_TIME = 0.5    # fresh time for one move
    global canvas

    if (len(sys.argv) == 8):
        MAZE_H = int(sys.argv[1])
        MAZE_W = int(sys.argv[2])
        AGENT = int(sys.argv[3])
        TREASURE = int(sys.argv[4])
        for w in sys.argv[5].split(','):
            try:
                if int(w)-1 <= MAZE_H*MAZE_W and int(w)-1 > 0:
                    WALLS.append(int(w)-1)
            except ValueError:
                pass
        for t in sys.argv[6].split(','):
            try:
                if int(t)-1 <= MAZE_H*MAZE_W and int(t)-1 > 0:
                    TRAPS.append(int(t)-1)
            except ValueError:
                pass
        FRESH_TIME = float(sys.argv[7])
        N_STATES = MAZE_H*MAZE_W  # Total of state
        WIDTH = MAZE_W*UNIT
        HEIGHT = MAZE_H*UNIT
    window = tk.Tk()
    # Centre la fenetre
    window.geometry("%dx%d%+d%+d" % (WIDTH,HEIGHT,(window.winfo_screenwidth()-WIDTH)//2,(window.winfo_screenheight()-HEIGHT)//2))
    window.title("The Treasure Hunt")
    canvas =  tk.Canvas(window,
                           height=MAZE_H * UNIT,
                           width=MAZE_W * UNIT)
    imageFond = Image.open("image/sol.png")
    imageFond = imageFond.resize((MAZE_W*UNIT, MAZE_H*UNIT), Image.ANTIALIAS)
    imageFond.save("image/resized.png", "png")
    canvas._photoFond = photoFond = ImageTk.PhotoImage(file= "image/resized.png")
    canvas.create_image(0,0,anchor=tk.NW,image=photoFond)

    origin = np.array([HALF_UNIT, HALF_UNIT])

    build_maze()
    rectangle=draw_rect(AGENT)
    #time.sleep(FRESH_TIME)
    canvas.delete(rectangle)
    
    q_table = rl(AGENT)
    print('\r\nQ-table:\n')
    print(q_table)

    window.mainloop()